//========================================================================================
//
//	File:				FWPriDeb.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFound.hpp"

// Need to include first so that FW_DEBUG is properly defined

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef FWPRISTR_H
#include "FWPriStr.h"
#endif

#ifndef FWPRIMEM_H
#include "FWPriMem.h"
#endif

#ifndef FWODFEXC_H
#include "FWODFExc.h"
#endif

#include <stdio.h>

#if defined(FW_BUILD_MAC) && !defined(__TYPES__)
#include <Types.h>
#endif

#if defined(THINK_CPLUS) && !defined(__PASCAL__)
#include <Pascal.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(_INC_WINDOWS)
#include <Windows.h>
#endif

#ifdef FW_BUILD_MAC
#pragma segment FWDebug
#endif


//----------------------------------------------------------------------------------------
// FW_CDebugConsole::RequireMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::RequireMessage(const char* message)
{
	// When debugging, we display the assertion and fail.  When not debugging, we 
	// just fail.  Users of FW_REQUIRE are advised that the FW_THROW can cross 
	// shared library boundaries.  
#ifdef FW_DEBUG
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoAssertMessage != 0 && message != 0)
		aConsole->fDoAssertMessage(aConsole, message);
#endif

	FW_Failure(FW_xUnknownError);
}

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::AssertMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::AssertMessage(const char* message)
{
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoAssertMessage != 0)
		aConsole->fDoAssertMessage(aConsole, message);
}
#endif

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::FatalMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::FatalMessage(const char* message)
{
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoFatalMessage != 0)
		aConsole->fDoFatalMessage(aConsole, message);
}
#endif

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::DebugMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::DebugMessage(const char* message)
{
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoDebugMessage != 0)
		aConsole->fDoDebugMessage(aConsole, message);
}
#endif

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::LogMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::LogMessage(const char* message)
{
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoLogMessage != 0)
		aConsole->fDoLogMessage(aConsole, message);
}
#endif

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::Debugger
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::Debugger()
{
	FW_SDebugConsole *aConsole = GetConsole();

	if (aConsole != 0 && aConsole->fDoDebugger != 0)
		aConsole->fDoDebugger(aConsole);
}
#endif

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// FW_CDebugConsole::SubclassResponsibility
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::SubclassResponsibility(const char* method)
{
	char msg[256];
	sprintf(msg, "Subclass responsibility: %s", method);

	DebugMessage(msg);
}
#endif


